import UIKit

/**
 Klasse zur Verwaltung von Autos für eine Waschstraße
 
- author: Klaus Reinold
- version: 1.0
*/
class Auto
{
    /** Das Kennzeichen */
    private var kennzeichen: String

    /**
     Konstruktor für Objekte der Klasse Auto
	- parameters:
		- kennzeichenNeu: Kennzeichen des Fahrzeugs
    */
    init(kennzeichenNeu: String)
    {
        kennzeichen = kennzeichenNeu
    }

    /**
	Gibt Information über das Auto aus.
    */
    func InformationAusgeben()
    {
        print(kennzeichen)
    }

    /**
     Gibt das Kennzeichen des Autos zurück
	- returns: Kennzeichen
    */
    func KennzeichenGeben() -> String
    {
        return kennzeichen
    }
}

/**
Beschreibung der Klasse Knoten

- author: Klaus Reinold
- version: 1.0
*/
class Knoten
{
    /** Verwaltung des nachfolgenden Knoten in der Warteschlange. */
    private var nachfolger: Knoten?
    /** Verwaltung der Daten des Autos */
    private var auto: Auto

    /**
	Konstruktor des Knoten
	- parameters:
		- autoNeu: verwaltetes Auto
    */
    init(autoNeu: Auto)
    {
        nachfolger  = nil
        auto = autoNeu
    }

    /**
	Nachfolger des Autos setzen
	- parameters:
		- nf: neuer Nachfolger
    */
    func NachfolgerSetzen(nf: Knoten)
    {
        nachfolger = nf
    }

    /**
	Gibt den Nachfolger des Knoten aus.
	- returns: Nachfolger
    */
    func NachfolgerGeben() -> Knoten?
    {
        return nachfolger
    }
    
    /**
	Gibt das Auto zurück
	- returns: Auto
    */
    func DatenelementGeben() -> Auto
    {
        return auto
    }
    
    /**
	Gibt Information über das Auto aus.
    */
    func InformationAusgeben()
    {
        auto.InformationAusgeben()
    }
}

/**
Beschreibung der Warteschlange

- author: Klaus Reinold
- version: 0.9
*/
class Warteschlange
{
    /** erster Knoten der Warteschlange */
    private var anfang: Knoten?
    /** letzter Knoten der Warteschlange */
    private var ende: Knoten?

    /**
    Konstruktor der Warteschlange
    */
    init()
    {
        anfang = nil
        ende = nil
    }

    /**
 	Fügt ein neues Auto am Ende der Warteschlange ein.
	- parameters:
		- autoNeu: neues Auto
    */
    func Einfügen(autoNeu: Auto)
    {
        let k = Knoten(autoNeu: autoNeu)
        if anfang == nil
        {
            anfang = k
        }
        else
        {
            ende!.NachfolgerSetzen(nf: k)
        }
        ende = k
    }

    /**
	Entfernt das erste Auto aus der Warteschlange und gibt ihn zurück.
	- returns: bisheriger erster Patient
    */
    func Entfernen() -> Auto?
    {
        let k = anfang
        if anfang != nil
        {
            if anfang === ende
            {
                anfang = nil
                ende = nil
            }
            else
            {
                anfang=anfang!.NachfolgerGeben()
            }
            return k!.DatenelementGeben()
        }
        else
        {
            return nil
        }
    }

    /**
	Gibt Information über die Warteschlange aus.
    */
    func InformationAusgeben()
    {
        if anfang != nil
        {
            anfang!.InformationAusgeben()
        }
    }
}
