import UIKit

/**
Beschreibung der Klasse Patient
 
- author: Klaus Reinold
- version: 1.0
*/
class Patient
{
    /** Name des Patienten */
    private var name: String

    /**
	Konstruktor des Patienten
	- parameters:
		- s: Name des Patienten
    */
    init(s: String)
    {
        name = s
    }

    /**
	Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        print(name)
    }

    /**
	Gibt den Namen des Patienten aus
	- returns: Name des Patienten
    */
    func NameGeben() -> String
    {
        return name
    }
}

/**
Beschreibung der Klasse Knoten

- author: Klaus Reinold
- version: 0.9
*/
class Knoten
{
    /** Verwaltung des nachfolgenden Knoten in der Warteschlange. */
    private var nachfolger: Knoten?
    /** Verwaltung der Daten des Patienten */
    private var daten: Patient

    /**
	Konstruktor des Knoten
	- parameters:
		- dNeu: Datenelement
    */
    init(dneu: Patient)
    {
        nachfolger  = nil
        daten = dneu
    }

    /**
	Nachfolger des Patienten setzen
	- parameters:
		- nf: neuer Nachfolger
    */
    func NachfolgerSetzen(nf: Knoten)
    {
        nachfolger = nf
    }

    /**
	Gibt den Nachfolger des Knoten aus.
	- returns: Nachfolger
    */
    func NachfolgerGeben() -> Knoten?
    {
        return nachfolger
    }
    
    /**
	Gibt das Datenelement zurück
	- returns: Datenelement
    */
    func DatenelementGeben() -> Patient
    {
        return daten
    }
    
    /**
	Gibt Information über den Patienten aus.
    */
    func InformationAusgeben()
    {
        daten.InformationAusgeben()
    }
}

/**
Beschreibung der Warteschlange

- author: Klaus Reinold
- version: 0.9
*/
class Warteschlange
{
    /** erster Patient der Warteschlange */
    private var anfang: Knoten?
    /** letzter Patient der Warteschlange */
    private var ende: Knoten?

    /**
    Konstruktor der Warteschlange
    */
    init()
    {
        anfang = nil
        ende = nil
    }

    /**
 	Fügt einen neuen Patienten am Ende der Warteschlange ein.
	- parameers:
		- patientNeu: neuer Patient
    */
    func Einfügen(patientNeu: Patient)
    {
        let k = Knoten(dneu: patientNeu)
        if anfang == nil
        {
            anfang = k
        }
        else
        {
            ende!.NachfolgerSetzen(nf: k)
        }
        ende = k
    }

    /**
	Entfernt den ersten Patient aus der Warteschlange und gibt ihn zurück.
	- returns: bisheriger erster Patient
    */
    func Entfernen() -> Patient?
    {
        let k = anfang
        if anfang != nil
        {
            if anfang === ende
            {
                anfang = nil
                ende = nil
            }
            else
            {
                anfang=anfang!.NachfolgerGeben()
            }
            return k!.DatenelementGeben()
        }
        else
        {
            return nil
        }
    }

    /**
	Gibt Information über die Warteschlange aus.
    */
    func InformationAusgeben()
    {
        if anfang != nil
        {
            anfang!.InformationAusgeben()
        }
    }
}
